%Code by Ke Gong
%This program may take a few minutes to run
clear;
clc;
close all;
dbstop if error
global R F ze T area

d_mid=[0.5 2.0 3.5].*1E-3;    %three cases for middle electrolyte thickness, unit: m
i_v=[0.1 0.2 0.3 0.5 1 2]; %current, unit: A
Q_mid_v=[400 300 200 100 75 50 25].*1E-6./60; %flow rate, unit: m^3/s

w_mid=4.6E-2; %middle electrolyte width, unit m
C_mid=2.5E3; %sodium/chloride ion concentration in middle, unit: mol/m^3

Q_pos=100.*1E-6./60; %flow rate of positive electrolyte, unit:m^3/s
d_pos=3.1E-3; %positive electrolyte thickness, unit:m
w_pos=3E-2; %positive electrolyte width, unit:m
C_pos=3.5E3; %sodium/chloride ion concentration in positive side, unit: mol/m^3

Q_neg=100.*1E-6./60; %flow rate of positive electrolyte, unit:m^3/s
d_neg=2E-3; %negative electrolyte thickness, unit:m
w_neg=3E-2; %negative electrolyte width, unit:m
C_neg=5.5E3; %sodium/chloride ion concentration in negative side, unit: mol/m^3

D_Na=1.33E-9; %diffusivity of sodium ion, unit: m^2/s
D_Cl=2.03E-9; %diffusivity of chloride ion, unit: m^2/s
D_NaCl=2*D_Na*D_Cl/(D_Na+D_Cl);
ze=1; %charge per ion
area=10E-4; %membrane/electrode area, unit m^2;
F=96480; %Faraday constant, unit: C/mol
R=8.314; %ideal gas constant, unit: J/mol/K
T=298; %temperature, unit: K



phiN=zeros(length(Q_mid_v),length(i_v),length(d_mid));
phiD=zeros(length(Q_mid_v),length(i_v),length(d_mid));

k=1;
for Q_mid=[400 300 200 100 75 50 25].*1E-6./60
    j=1;
    for i=[0.1 0.2 0.3 0.5 1 2]
        l=1;
        for d_mid=[0.5 2.0 3.5].*1E-3
            %Calculate limiting current of CEM and AEM 
            ilim_CEM_neg=limitcurrent(D_NaCl,w_neg,d_neg,Q_neg,C_neg);
            ilim_AEM_pos=limitcurrent(D_NaCl,w_pos,d_pos,Q_pos,C_pos);
            ilim_CEM_mid=limitcurrent(D_NaCl,w_mid,d_mid,Q_mid,C_mid);
            ilim_AEM_mid=limitcurrent(D_NaCl,w_mid,d_mid,Q_mid,C_mid);
           
            %Calculate Donnan potential across CEM and AEM
            phiD_CEM=phiDonnan(i,ilim_CEM_neg,C_neg,ilim_CEM_mid,C_mid);
            phiD_AEM=phiDonnan(i,ilim_AEM_mid,C_mid,ilim_AEM_pos,C_pos);
           
            phiD(k,j,l)=phiD_CEM+phiD_AEM;
            RcpD(k,j,l)=phiD(k,j,l)/i;
                        
        l=l+1;
        end
    j=j+1;   
    end
k=k+1;  
end

%Calculate RcpN
fullParamSpace3;
load RcpN_MIT

%plot total area-specific Rcp (ASRcpT) at different thicknesses
i_plot=[10 20 30 50 100 200];
q_plot=[400 300 200 100 75 50 25];
[I_plot,Q_plot]=meshgrid(i_plot,q_plot);
ASRcpN_mit=RcpN*area*1E4; %Convert to area specific resistance, unit: ohm.cm^2
ASRcpD=RcpD*area*1E4;
ASRcpT_mit=ASRcpD+ASRcpN_mit;
spec={'ro','b+','gd'};
n=1;
for d_mid=[0.5 2.0 3.5].*1E-3
    X(:,:,n)=(Q_plot*1E-6/60/d_mid/w_mid)./(I_plot*1E-3/1E-4/F/C_mid);
    subplot(2,2,1)
    loglog(X(:,:,n),ASRcpT_mit(:,:,n),spec{n})
    axis([10,1E6,0.01,10])
    hold on
    subplot(2,2,n+1)
    surf(I_plot,Q_plot,ASRcpT_mit(:,:,n));
    colormap hsv
    set(gca,'YDir','rev')
    n=n+1;
end

%Convert matrix data for plot in Origin
orinplot_d05=[m2v(X(:,:,1)),m2v(ASRcpT_mit(:,:,1))];
orinplot_d20=[m2v(X(:,:,2)),m2v(ASRcpT_mit(:,:,2))];
orinplot_d35=[m2v(X(:,:,3)),m2v(ASRcpT_mit(:,:,3))];


